# -*- coding: utf-8 -*-
# Copyright (c) 2015 Spotify AB
from __future__ import absolute_import, division, print_function

import os

PAR_DIR = os.path.abspath(os.path.dirname(__file__))
EXAMPLES = os.path.join(PAR_DIR + '/data/examples/')
VALIDATE = os.path.join(PAR_DIR + '/data/validate/')
FIXTURES = os.path.join(PAR_DIR + '/data/fixtures/')
UPDATE = os.path.join(PAR_DIR + '/data/update/')
JSONREF = os.path.join(PAR_DIR + '/data/jsonref/')
