@import url("rtdtheme.css");
@import url("badge_only.css");

.wy-side-nav-search {
    background-color: #1ED760;
}

.wy-side-nav-search input[type=text] {
    width: 100%;
    border-radius: 50px;
    padding: 6px 12px;
    border-color: #1ED760
}

.rst-content a {
    color: #1ED760;
}

.rst-content dl:not(.docutils) dt {
    padding: 2px;
}

dt#ramlfications.raml.RAMLRoot.resources {
    padding: 2px;
}


code, .rst-content tt {
    color: #1ED760;
}

.descclassname, .descname {
    background: none;
    border: none;
    padding: 0;
    font-size: 100% !important;
    color: #000;
    font-weight: bold;
}

.fa-home:before,
.icon-home:before {
    content: "\f1bc"
}

.wy-nav-content {
    padding: 1.618em 3.236em;
    height: 100%;
    max-width: 900px;
    margin: auto
}
